"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const modification_1 = require("../../../parsers/w3x/w3u/modification");
const modifiedobject_1 = require("../../../parsers/w3x/w3u/modifiedobject");
class Objects {
    constructor(type, metaData, data) {
        this.objects = new Map();
        this.type = type;
        this.metaData = metaData;
        this.data = data;
    }
    load(originalTable, customTable) {
        for (const { oldId, newId, modifications } of originalTable.objects) {
            this.objects.set(oldId, new this.type(this, oldId, newId, modifications));
        }
        for (const { oldId, newId, modifications } of customTable.objects) {
            this.objects.set(oldId, new this.type(this, oldId, newId, modifications));
        }
    }
    save(originalTable, customTable) {
        for (const object of this.objects.values()) {
            if (object.modifications.size) {
                const objectMod = new modifiedobject_1.default();
                for (const [modId, modValue] of object.modifications) {
                    const mod = new modification_1.default();
                    mod.id = modId;
                    mod.value = modValue;
                    objectMod.modifications.push(mod);
                }
                objectMod.oldId = object.oldId;
                objectMod.newId = object.newId;
                if (object.newId === '\0\0\0\0') {
                    originalTable.objects.push(objectMod);
                }
                else {
                    customTable.objects.push(objectMod);
                }
            }
        }
    }
    copy(baseId) {
        const id = 'asd1';
        const object = new this.type(this, baseId, id, []);
        // If the base object has any modifications, copy them over.
        const baseObject = this.objects.get(id);
        if (baseObject) {
            for (const [key, value] of baseObject.modifications) {
                object.modifications.set(key, value);
            }
        }
        this.objects.set(id, object);
        return object;
    }
    get(id) {
        let object = this.objects.get(id);
        // If this object exists in the base data, get it and add it to the objects list in case it is modified.
        if (!object && !!this.data.getRow(id)) {
            object = new this.type(this, id, '\0\0\0\0', []);
            this.objects.set(id, object);
        }
        return object;
    }
}
exports.default = Objects;
//# sourceMappingURL=objects.js.map